@@ -43,7 +43,7 @@ class ProxyHandler:
         headers = handler_context.get_request_headers()
 
         r = requests.request(method, origin_url, headers=headers, data=data, cookies=handler_context.request.cookies,
-                            stream=True, verify=False)
+                            stream=True, verify=False, allow_redirects=False)
 
         # 增加数据源标记，此数据经代理得到
         resp_headers = [('lyrebird', 'proxy')]
